// TestPostGresODBC.cpp : Defines the entry point for the console application.
//
// 
// I am using a very simple table for this test.  Here are the details:
// Database name = "alex"
// User = "alex", password is blank
// Table name = "thedata"
// ODBC DSN = "AlexTest"
// 2 columns in the table - "text1" and "text2", both are VARCHAR, len = 64
// 

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <sql.h>
#include <sqlext.h>

//-- Globals
#define PT_BUF_LEN 100
#define PT_NUM_COLS 2
#define PT_HARNESS_TITLE "Postgres odbc test"

//#define PT_DESCRIBE_BIND_COL   //leave this out, don't need it to see issue

HENV g_henv = 0;
HDBC g_hdbc = 0;
SQLHANDLE g_stmt = NULL;
char* g_pszData = NULL;

#ifdef PT_DESCRIBE_BIND_COL
   SQLCHAR* g_pszBuff[PT_NUM_COLS];
   SQLINTEGER g_nBuffLen[PT_NUM_COLS];
#endif


//-- Our util functions
int doLogin();
int doLogout();
int doQuery();
int doColumnInfo();


/*-----------------------------------------------------------------------------
 * Name:       main
 * Purpose:    Entry point for test harness
 *
 *
 * Returns:    0
 *----------------------------------------------------------------------------*/
int main(int argc, char* argv[])
{
 	int rc;
   bool bSuccess = true;

   // Login
   rc = doLogin();
   if (rc != SQL_SUCCESS && rc!= SQL_SUCCESS_WITH_INFO)
   {
      MessageBox(NULL, "doLogin failed", PT_HARNESS_TITLE, MB_OK);
      return 0;
   }

   // Try a query
   rc = doQuery();
   if (rc != SQL_SUCCESS && rc!= SQL_SUCCESS_WITH_INFO)
   {
      MessageBox(NULL, "doQuery failed", PT_HARNESS_TITLE, MB_OK);
      bSuccess = false;
      // fall through to logout
   }

   // Logout
   rc = doLogout();
   if (rc != SQL_SUCCESS && rc!= SQL_SUCCESS_WITH_INFO)
   {
      MessageBox(NULL, "doLogout failed", PT_HARNESS_TITLE, MB_OK);
      return 0;
   }

   if(bSuccess)
      MessageBox(NULL, "Success", PT_HARNESS_TITLE, MB_OK);
	
   return 0;
}

int doLogin()
{
  char username[256];
  char passwd[256];
  char tnsname[256];

  int status;

   strcpy(username, "alex");
   strcpy(tnsname, "AlexTest");
   strcpy(passwd, "");

  if(g_henv==0||g_hdbc==0){
   status=SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&g_henv);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   status=SQLSetEnvAttr(g_henv,SQL_ATTR_ODBC_VERSION,(void*)SQL_OV_ODBC3,SQL_NTS);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   status=SQLAllocHandle(SQL_HANDLE_DBC,g_henv,&g_hdbc);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;
  }

  status=SQLConnect
    (g_hdbc, 
     (unsigned char*)tnsname,SQL_NTS,
     (unsigned char*)username,SQL_NTS,
     (unsigned char*)passwd,SQL_NTS); 

   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   status=SQLSetConnectAttr
    (g_hdbc,
     SQL_ATTR_AUTOCOMMIT,
     (SQLPOINTER)SQL_AUTOCOMMIT_OFF,
     SQL_NTS); 

   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   return SQL_SUCCESS;
}

int doLogout()
{
   int status;

   status=SQLDisconnect(g_hdbc); 
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   status = SQLFreeHandle(SQL_HANDLE_DBC, g_hdbc);
   status = SQLFreeHandle(SQL_HANDLE_ENV, g_henv);

#ifdef PT_DESCRIBE_BIND_COL
   for(int i = 0; i < PT_NUM_COLS; i++)
   {
      if(g_pszBuff[i] != NULL)
         delete [] g_pszBuff[i];
   }
#endif

   return SQL_SUCCESS;
}

int doQuery()
{
   int rc;
   int status;
   SQLINTEGER nStrLen = SQL_NTS;
   SQLUINTEGER crow, array;

   g_pszData = new char[PT_BUF_LEN];
   memset(g_pszData, 0, PT_BUF_LEN);

   status=SQLAllocHandle(SQL_HANDLE_STMT, g_hdbc, &g_stmt);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   SQLCHAR* myPs;
   myPs = new SQLCHAR [100];
   strcpy((char *) myPs, "SELECT * from thedata WHERE text1 = ?");
   SQLINTEGER nLen = strlen((const char *)myPs);

   status = SQLPrepare(g_stmt, myPs, nLen);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;
   
   //-- Bind our global char buffer to "text1" column
   status = SQLBindParameter(g_stmt,
                    1,
                    SQL_PARAM_INPUT,
                    SQL_C_CHAR,
                    SQL_VARCHAR,
                    64,
                    0, //ignore decimal digits
                    (SQLPOINTER) g_pszData,
                    (SQLINTEGER) PT_BUF_LEN,
                    (SQLINTEGER*) &nStrLen);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;


   //-- Do the column info querying
   rc = doColumnInfo();
   if (rc != SQL_SUCCESS && rc!= SQL_SUCCESS_WITH_INFO)
   {
      MessageBox(NULL, "doColumnInfo failed", PT_HARNESS_TITLE, MB_OK);
      return rc;
   }


   array = 0;
   crow = 0;
   status=SQLSetStmtAttr(g_stmt,SQL_ATTR_ROW_ARRAY_SIZE,&array,SQL_NTS);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;
   status=SQLSetStmtAttr(g_stmt,SQL_ATTR_ROWS_FETCHED_PTR,&crow,SQL_NTS); 
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;


   status = SQLExecute(g_stmt);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   status=SQLFetchScroll(g_stmt, SQL_FETCH_NEXT, 1);
   if(status==SQL_ERROR||status==SQL_INVALID_HANDLE)
      return status;
   if(status==SQL_NO_DATA_FOUND)
      return status;


   TCHAR szTestBuff[256];
   sprintf(szTestBuff, "Rows fetched ==>%d", (int) crow);
   MessageBox(NULL, szTestBuff, PT_HARNESS_TITLE, MB_OK);


   delete [] myPs;
   delete [] g_pszData;
   
   status = SQLFreeHandle(SQL_HANDLE_STMT, g_stmt);
   if(status!=SQL_SUCCESS)
      return status;

   return SQL_SUCCESS;
}

int doColumnInfo()
{
   int status;
   SQLSMALLINT nCol;


   /*****************************************
    * If we set the buffer that we binded to column "text1" now, it will work
    * and the correct number of rows will be queried in our SQLExecute call in
    * doQuery().
    *****************************************/
   //strcpy(g_pszData, "testdata");

   status = SQLNumResultCols(g_stmt, &nCol);
   if(status!=SQL_SUCCESS&&status!=SQL_SUCCESS_WITH_INFO)
      return status;

   /*****************************************
    * ERROR HAPPENS HERE:
    * If we set the buffer that we binded to column "text1" now, it will FAIL.
    * The SQLExecute in doQuery() will succeed (finding 0 rows), and the 
    * SQLFetchScroll call will return SQL_NO_DATA_FOUND.
    *
    * This is because of a bug in the odbc driver. The call to SQLNumResultCols
    * will execute the query in the driver, but the buffer that is binded is 
    * empty, so no rows will be found.  This of course assumes that the table
    * doesn't contain any empty records for the "text1" column, which is correct
    * for our test.  Then when we call SQLExcute, the driver will realize it has
    * already done the query work and will not do it again, even though our 
    * buffer for parameter substitution has changed (next line of code below 
    * this comment).
    *****************************************/
   strcpy(g_pszData, "testdata");
   
#ifdef PT_DESCRIBE_BIND_COL
   SQLSMALLINT nNameLen, nDataType;
   SQLSMALLINT nDigits, nNullable;
   SQLCHAR* pszColName = new SQLCHAR [100];
   SQLUINTEGER uColSize;

   for(int i = 1; i <= nCol; i++)
   {
      nNameLen = 0;
      nDataType = 0;
      uColSize = 0;
      nDigits = 0;
      nNullable = 0;
      status = SQLDescribeCol(g_stmt,
                     (SQLSMALLINT) i,
                     (SQLCHAR *) pszColName,
                     (SQLSMALLINT) 100,
                     (SQLSMALLINT*) &nNameLen,
                     (SQLSMALLINT*) &nDataType,
                     (SQLUINTEGER *) &uColSize,
                     (SQLSMALLINT*) &nDigits,
                     (SQLSMALLINT*) &nNullable);

      g_nBuffLen[i-1] = SQL_NTS;
      g_pszBuff[i-1] = new SQLCHAR[PT_BUF_LEN];//these are deleted in disconnect
      status = SQLBindCol( g_stmt,
                           (SQLSMALLINT) i,
                           SQL_C_CHAR,
                           (SQLPOINTER) g_pszBuff[i-1],
                           (SQLINTEGER) PT_BUF_LEN,
                           (SQLINTEGER*) &g_nBuffLen[i-1]);
   }
   //-- Free up
   delete [] pszColName;

#endif   


   return SQL_SUCCESS;
}

